@@ -1,7 +1,7 @@
 /* -*- indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8; -*- */
 
 /* Copyright (c) 2004 - 2006 Derek Foreman, Ben Jansens
-   Copyright (c) 2006 - 2014 Thomas Schmitt <scdbackup@gmx.net>
+   Copyright (c) 2006 - 2016 Thomas Schmitt <scdbackup@gmx.net>
    Provided under GPL version 2 or later.
 */
 
@@ -26,6 +26,12 @@
 #include <pthread.h>
 #include <errno.h>
 #include <fcntl.h>
+
+/* ts B41126 : O_BINARY is needed for Cygwin but undefined elsewhere */
+#ifndef O_BINARY
+#define O_BINARY 0
+#endif
+
 #include "libburn.h"
 #include "init.h"
 #include "drive.h"
@@ -52,6 +58,9 @@
 /* A90815 : for mmc_obtain_profile_name() */
 #include "mmc.h"
 
+/* B60730 : for Libburn_do_no_immed_defaulT */
+#include "os.h"
+
 #include "libdax_msgs.h"
 extern struct libdax_msgs *libdax_messenger;
 
@@ -77,10 +86,23 @@ int burn_setup_drive(struct burn_drive *d, char *fname)
 	d->released = 1;
 	d->stdio_fd = -1;
 	d->status = BURN_DISC_UNREADY;
+	d->erasable = 0;
+	d->current_profile = -1;
 	d->do_stream_recording = 0;
         d->stream_recording_start= 0;
 	d->role_5_nwa = 0;
+
+#ifdef Libburn_do_no_immed_defaulT
+	d->do_no_immed = Libburn_do_no_immed_defaulT;
+#else
+	d->do_no_immed = 0;
+#endif
+
 	d->features = NULL;
+	d->drive_serial_number = NULL;
+	d->drive_serial_number_len = -1;
+	d->media_serial_number = NULL;
+	d->media_serial_number_len = -1;
 	return 1;
 }
 
@@ -106,6 +128,10 @@ void burn_drive_free_subs(struct burn_drive *d)
 		close (d->stdio_fd);
 	d->stdio_fd = -1;
 	burn_feature_descr_free(&(d->features), 0);
+	BURN_FREE_MEM(d->drive_serial_number);
+	BURN_FREE_MEM(d->media_serial_number);
+        d->drive_serial_number = d->media_serial_number = NULL;
+	d->drive_serial_number_len = d->media_serial_number_len = 0;
 	sg_dispose_drive(d, 0);
 }
 
@@ -266,53 +292,7 @@ int burn_drive_inquire_media(struct burn_drive *d)
 	    (d->mdata->p2a_valid > 0 &&
 		(d->mdata->cdr_write || d->mdata->cdrw_write ||
 		 d->mdata->dvdr_write || d->mdata->dvdram_write)) ) {
-
-#define Libburn_knows_correct_state_after_loaD 1
-#ifdef Libburn_knows_correct_state_after_loaD
-
 		d->read_disc_info(d);
-
-#else
-		/* ts A61227 : This repeated read_disc_info seems
-		               to be obsoleted by above d->getcaps(d).
-		*/
-		/* ts A60908 */
-		/* Trying to stabilize the disc status after eventual load
-		   without closing and re-opening the drive */
-		/* This seems to work for burn_disc_erasable() .
-		   Speed values on RIP-14 and LITE-ON 48125S are stable
-		   and false, nevertheless. */
-		int was_equal = 0, must_equal = 3, max_loop = 20;
-		int loop_count, old_speed = -1234567890, new_speed= -987654321;
-		int old_erasable = -1234567890, new_erasable = -987654321;
-
-		fprintf(stderr,"LIBBURN_DEBUG: read_disc_info()\n");
-		for (loop_count = 0; loop_count < max_loop; loop_count++){
-			old_speed = new_speed;
-			old_erasable = new_erasable;
-
-			d->read_disc_info(d);
-			if(d->status == BURN_DISC_UNSUITABLE)
-		break;
-
-			new_speed = burn_drive_get_write_speed(d);
-			new_erasable = burn_disc_erasable(d);
-		        if (new_speed == old_speed &&
-			    new_erasable == old_erasable) {
-				was_equal++;
-				if (was_equal >= must_equal)
-		break;
-			} else
-				was_equal = 0;
-			/*
-			if (loop_count >= 1 && was_equal == 0)
-			*/
-				fprintf(stderr,"LIBBURN_DEBUG: %d : speed %d:%d   erasable %d:%d\n",
-					loop_count,old_speed,new_speed,old_erasable,new_erasable);
-			usleep(100000);
-		}
-#endif /* ! Libburn_knows_correct_state_after_loaD */
-
 	} else {
 		if (d->current_profile == -1 || d->current_is_cd_profile)
 			d->read_toc(d);
@@ -449,6 +429,7 @@ int burn_drive_grab_stdio(struct burn_drive *d, int flag)
 		/* despite its name : last valid address, not size */
 		d->media_read_capacity =
 					read_size / 2048 - !(read_size % 2048);
+		d->mr_capacity_trusted = 1;
 		if ((stat_ret == -1 || is_rdwr) && d->devname[0]) { 
 		       	ret = burn_os_stdio_capacity(d->devname, 0, &size);
 			if (ret > 0)
@@ -742,6 +723,14 @@ int burn_drive_release_fl(struct burn_drive *d, int flag)
 	}
 
 	d->needs_sync_cache = 0; /* just to be sure */
+
+	if (d->drive_serial_number != NULL)
+		BURN_FREE_MEM(d->drive_serial_number);
+	if (d->media_serial_number != NULL)
+		BURN_FREE_MEM(d->media_serial_number);
+	d->drive_serial_number = d->media_serial_number = NULL;
+	d->drive_serial_number_len = d->media_serial_number_len = 0;
+
 	d->released = 1;
 
 	/* ts A61125 : outsourced model aspects */
@@ -909,6 +898,12 @@ void burn_disc_erase_sync(struct burn_drive *d, int fast)
 #else /* Libburn_old_progress_looP */
 
 	while (1) {
+
+		/* >>> ??? ts B60730 : abort if user interrupts ?
+		if (d->cancel)
+	break;
+		*/
+
 		ret = d->get_erase_progress(d);
 		if (ret == -2 || ret > 0)
 	break;
@@ -917,6 +912,12 @@ void burn_disc_erase_sync(struct burn_drive *d, int fast)
 		sleep(1);
 	}
 	while (1) {
+
+		/* >>> ??? ts B60730 : abort if user interrupts ?
+		if (d->cancel)
+	break;
+		*/
+
 		ret = d->get_erase_progress(d);
 		if(ret == -2)
 	break;
@@ -933,7 +934,7 @@ void burn_disc_erase_sync(struct burn_drive *d, int fast)
 
 	/* ts A61125 : update media state records */
 	burn_drive_mark_unready(d, 0);
-	if (d->drive_role == 1)
+	if (d->drive_role == 1 && !d->cancel)
 		burn_drive_inquire_media(d);
 	d->busy = BURN_DRIVE_IDLE;
 	if (was_error)
@@ -1197,24 +1198,21 @@ void burn_drive_cancel(struct burn_drive *d)
 }
 
 
-static void strip_spaces(char *str)
+static void strip_spaces(char *str, size_t len)
 {
-	char *tmp;
+	char *tmp, *tmp2;
 
-	tmp = str + strlen(str) - 1;
-	while (isspace(*tmp))
-		*(tmp--) = '\0';
-
-	tmp = str;
-	while (*tmp) {
+	/* Remove trailing blanks */
+	for (tmp = str + len - 1; tmp >= str && (isspace(*tmp) || !*tmp); tmp--)
+		*tmp = 0;
+	/* Condense remaining blank intervals to single blanks */
+	for (tmp = str; tmp < str + len - 1 && *tmp; tmp++) {
 		if (isspace(*tmp) && isspace(*(tmp + 1))) {
-			char *tmp2;
-
-			for (tmp2 = tmp + 1; *tmp2; ++tmp2)
+			for (tmp2 = tmp + 1; tmp2 < str + len && *tmp2; tmp2++)
 				*(tmp2 - 1) = *tmp2;
 			*(tmp2 - 1) = '\0';
-		} else
-			++tmp;
+			tmp--; /* try same first blank again */
+		}
 	}
 }
 
@@ -1236,11 +1234,11 @@ static int drive_getcaps(struct burn_drive *d, struct burn_drive_info *out)
 	id = (struct burn_scsi_inquiry_data *)d->idata;
 
 	memcpy(out->vendor, id->vendor, sizeof(id->vendor));
-	strip_spaces(out->vendor);
+	strip_spaces(out->vendor, sizeof(id->vendor));
 	memcpy(out->product, id->product, sizeof(id->product));
-	strip_spaces(out->product);
+	strip_spaces(out->product, sizeof(id->product));
 	memcpy(out->revision, id->revision, sizeof(id->revision));
-	strip_spaces(out->revision);
+	strip_spaces(out->revision, sizeof(id->revision));
 	strncpy(out->location, d->devname, 16);
 	out->location[16] = '\0';
 
@@ -1523,7 +1521,7 @@ void burn_drive_info_free(struct burn_drive_info drive_infos[])
 
 	/* ts A60904 : This looks a bit weird. [ts A70907 : not any more]
 	   burn_drive_info is not the manager of burn_drive but only its
-	   spokesperson. To my knowlege drive_infos from burn_drive_scan()
+	   spokesperson. To my knowledge drive_infos from burn_drive_scan()
 	   are not memorized globally. */
 	free((void *) drive_infos);
 
@@ -1531,7 +1529,7 @@ void burn_drive_info_free(struct burn_drive_info drive_infos[])
 	/* ts A70903 : THIS IS WRONG !      (disabled now)
 	   It endangers multi drive usage.
 	   This call is not entitled to delete all drives, only the
-	   ones of the array which it recieves a parmeter.
+	   ones of the array which it receives a parmeter.
 
 	   Problem:  It was unclear how many items are listed in drive_infos
            Solution: Added a end marker element to any burn_drive_info array 
@@ -1685,17 +1683,17 @@ static int burn_role_by_access(char *fname, int flag)
 #endif
 	int fd;
         
-	fd = open(fname, O_RDWR | O_LARGEFILE);
+	fd = open(fname, O_RDWR | O_LARGEFILE | O_BINARY);
 	if (fd != -1) {
 		close(fd);
 		return 2;
 	}
-	fd = open(fname, O_RDONLY | O_LARGEFILE);
+	fd = open(fname, O_RDONLY | O_LARGEFILE | O_BINARY);
         if (fd != -1) {
 		close(fd);
 		return 4;
 	}
-	fd = open(fname, O_WRONLY | O_LARGEFILE);
+	fd = open(fname, O_WRONLY | O_LARGEFILE | O_BINARY);
 	if (fd != -1) {
 		close(fd);
 		return 5;
@@ -1807,10 +1805,12 @@ int burn_drive_grab_dummy(struct burn_drive_info *drive_infos[], char *fname)
 		strcpy(d->current_profile_text,"stdio file");
 		d->current_is_cd_profile = 0;
 		d->current_is_supported_profile = 1;
-		if (read_size >= 0)
+		if (read_size >= 0) {
 			/* despite its name : last valid address, not size */
 			d->media_read_capacity =
 				read_size / 2048 - !(read_size % 2048);
+			d->mr_capacity_trusted = 1;
+		}
 		burn_drive_set_media_capacity_remaining(d, size);
 	} else
 		d->current_profile = 0; /* Drives return this if empty */
@@ -1855,7 +1855,7 @@ ex:;
 
 
 /* ts A60823 */
-/** Aquire a drive with known persistent address. 
+/** Acquire a drive with known persistent address. 
 */
 int burn_drive_scan_and_grab(struct burn_drive_info *drive_infos[], char* adr,
 			     int load)
@@ -1863,7 +1863,7 @@ int burn_drive_scan_and_grab(struct burn_drive_info *drive_infos[], char* adr,
 	unsigned int n_drives;
 	int ret, i;
 
-	/* check wether drive adress is already registered */
+	/* check wether drive address is already registered */
 	for (i = 0; i <= drivetop; i++)
 		if (drive_array[i].global_index >= 0)
 			if (strcmp(drive_array[i].devname, adr) == 0)
@@ -1922,14 +1922,15 @@ int burn_drive_adr_debug_msg(char *fmt, char *arg)
 	int ret;
 	char *msg = NULL, *msgpt;
 
-	BURN_ALLOC_MEM(msg, char, 4096);
-	msgpt = msg;
-	if(arg != NULL)
-		sprintf(msg, fmt, arg);
-	else
-		msgpt = fmt;
 	if(libdax_messenger == NULL)
 		return 0;
+	if(arg != NULL) {
+		BURN_ALLOC_MEM(msg, char, 4096);
+		msgpt = msg;
+		sprintf(msg, fmt, arg);
+	} else {
+		msgpt = fmt;
+	}
 	ret = libdax_msgs_submit(libdax_messenger, -1, 0x00000002,
 				LIBDAX_MSGS_SEV_DEBUG, LIBDAX_MSGS_PRIO_ZERO,
 				msgpt, 0, 0);
@@ -2906,7 +2907,7 @@ int burn_disc_get_multi_caps(struct burn_drive *d, enum burn_write_types wt,
 		size = d->media_capacity_remaining;
 		burn_os_stdio_capacity(d->devname, 0, &size);
 		burn_drive_set_media_capacity_remaining(d, size);
-		o->start_range_high = d->media_capacity_remaining;
+		o->start_range_high = d->media_capacity_remaining - 2048;
 		o->start_alignment = 2048; /* imposting a drive, not a file */
 		o->might_do_sao = 4;
 		o->might_do_tao = 2;
@@ -2921,7 +2922,7 @@ int burn_disc_get_multi_caps(struct burn_drive *d, enum burn_write_types wt,
 
 		/* >>> start_range_low = file size rounded to 2048 */;
 
-		o->start_range_high = d->media_capacity_remaining;
+		o->start_range_high = d->media_capacity_remaining - 2048;
 		o->start_alignment = 2048; /* imposting a drive, not a file */
 		if (s == BURN_DISC_APPENDABLE) {
 			if (wt == BURN_WRITE_SAO || wt == BURN_WRITE_RAW)
@@ -3002,7 +3003,7 @@ int burn_disc_get_multi_caps(struct burn_drive *d, enum burn_write_types wt,
 					&num_formats);
 		if (ret == 1) {
 			if (status == BURN_FORMAT_IS_FORMATTED)
-				o->start_range_high = size;
+				o->start_range_high = size - 2048;
 			if (d->current_profile == 0x13) {
 				o->start_alignment = 32 * 1024;
 				for (i = 0; i < num_formats; i++) {
@@ -3238,7 +3239,7 @@ int burn_drive_equals_adr(struct burn_drive *d1, char *adr2_in, int role2)
 		if (stat_ret1 == -1 || stat_ret2 == -1) {
 			if (stat_ret1 != -1 || stat_ret2 != -1)
 				 {ret = 0; goto ex;}
-					/* one adress existing, one not */
+					/* one address existing, one not */
 
 			/* Two non-existing file objects */
 
@@ -3346,7 +3347,8 @@ int burn_drive_set_media_capacity_remaining(struct burn_drive *d, off_t value)
 /* ts A81215 : API */
 int burn_get_read_capacity(struct burn_drive *d, int *capacity, int flag)
 {
-	*capacity = d->media_read_capacity + 1;
+	*capacity = d->media_read_capacity +
+			(d->media_read_capacity != 0x7fffffff);
 	return (d->media_read_capacity != 0x7fffffff);
 }
 
@@ -3473,7 +3475,7 @@ int burn_feature_descr_new(struct burn_feature_descr **new,
 	*new = NULL;
 	if (descr_len < 4)
 		return 0;
-	(*new) = o = calloc(1, sizeof(struct burn_speed_descriptor));
+	(*new) = o = calloc(1, sizeof(struct burn_feature_descr));
 	if (o == NULL)
 		return -1;
 	o->feature_code = (descr[0] << 8) | descr[1];
@@ -3531,3 +3533,83 @@ int burn_drive_has_feature(struct burn_drive *d, int feature_code,
 }
 
 
+/* ts B51016 API */
+int burn_drive_get_serial_no(struct burn_drive *d, char **sno, int *sno_len)
+{
+	int ret;
+
+	if (*sno != NULL)
+		BURN_FREE_MEM(*sno);
+	if (d->drive_serial_number_len > 0)
+		*sno_len = d->drive_serial_number_len;
+	else
+		*sno_len = 0;
+	BURN_ALLOC_MEM(*sno, char, *sno_len + 1);
+	if (d->drive_serial_number_len > 0)
+		memcpy(*sno, d->drive_serial_number, *sno_len);
+	(*sno)[*sno_len] = 0;
+	ret = 1;
+ex:
+	return ret;
+}
+
+
+/* ts B51016 API */
+int burn_drive_get_media_sno(struct burn_drive *d, char **sno, int *sno_len)
+{
+	int ret;
+
+#ifdef Libburn_enable_scsi_cmd_ABh
+	struct burn_feature_descr *feat;
+#endif
+
+	if (*sno != NULL)
+		BURN_FREE_MEM(*sno);
+	*sno = NULL;
+
+	if (d->media_serial_number_len == -1) {
+
+#ifdef Libburn_enable_scsi_cmd_ABh
+
+		if (burn_drive_has_feature(d, 0x109, &feat, 0))
+
+#ifndef Libburn_enable_scsi_cmd_ABh_pretend_currenT
+			if (feat->flags & 1) /* current */
+#endif
+
+				spc_read_media_serial_number(d);
+
+#else
+		;
+
+#endif /* ! Libburn_enable_scsi_cmd_ABh */
+
+	}
+
+	if (d->media_serial_number_len > 0)
+		*sno_len = d->media_serial_number_len;
+	else
+		*sno_len = 0;
+	BURN_ALLOC_MEM(*sno, char, *sno_len + 1);
+	if (*sno_len > 0)
+		memcpy(*sno, d->media_serial_number, *sno_len);
+	(*sno)[*sno_len] = 0;
+	ret = 1;
+ex:
+	return ret;
+}
+
+
+int burn_drive_set_immed(struct burn_drive *drive, int enable)
+{
+	drive->do_no_immed = !enable;
+	return 1;
+}
+
+
+int burn_drive_get_immed(struct burn_drive *drive)
+{
+	return !drive->do_no_immed;
+}
+
+
