@@ -1,7 +1,7 @@
 /* -*- indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8; -*- */
 
 /* Copyright (c) 2004 - 2006 Derek Foreman, Ben Jansens
-   Copyright (c) 2006 - 2012 Thomas Schmitt <scdbackup@gmx.net>
+   Copyright (c) 2006 - 2016 Thomas Schmitt <scdbackup@gmx.net>
    Provided under GPL version 2 or later.
 */
 
@@ -38,6 +38,11 @@
 #include <sys/stat.h>
 #include <sys/time.h>
 
+/* ts B41126 : O_BINARY is needed for Cygwin but undefined elsewhere */
+#ifndef O_BINARY
+#define O_BINARY 0
+#endif
+
 #include "error.h"
 #include "sector.h"
 #include "libburn.h"
@@ -86,6 +91,10 @@ extern struct libdax_msgs *libdax_messenger;
 */
 #define Libburn_bd_streamed_obS (64 * 1024)
 
+/* The number of retries if write(2) returns a short, non-negative write count.
+*/
+#define Libburn_stdio_write_retrieS 16
+
 
 static int type_to_ctrl(int mode)
 {
@@ -217,8 +226,6 @@ int burn_write_track_minsize(struct burn_write_opts *o, struct burn_session *s,
 			0x0002011a,
 			LIBDAX_MSGS_SEV_NOTE, LIBDAX_MSGS_PRIO_HIGH, msg,0,0);
 		step = BUFFER_SIZE / 4096; /* shall fit any sector size */
-		if (step <= 0)
-			step = 1;
 		seclen = burn_sector_length(t->mode);
 		if (seclen <= 0)
 			seclen = 2048;
@@ -782,7 +789,7 @@ XXX this is untested :)
 			rem += burn_track_get_shortage(tar[i]);
 
 			/* ts A61101 : I doubt that linking would yield a
-					desireable effect. With TAO it is
+					desirable effect. With TAO it is
 					counterproductive in any way.
 			*/
 			if (o->write_type == BURN_WRITE_TAO)
@@ -1101,7 +1108,7 @@ int burn_write_track(struct burn_write_opts *o, struct burn_session *s,
 	struct burn_drive *d = o->drive;
 	int i, tmp = 0, open_ended = 0, ret= 0, nwa, lba;
 	int sectors;
-	char msg[80];
+	char msg[160];
 
 	d->rlba = -150;
 
@@ -1791,7 +1798,8 @@ static int transact_dvd_chunk(struct burn_write_opts *opts,
 	static int tee_fd= -1;
 	if(tee_fd==-1)
 		tee_fd= open("/tmp/libburn_sg_readin",
-				O_WRONLY|O_CREAT|O_TRUNC,S_IRUSR|S_IWUSR);
+				O_WRONLY | O_CREAT | O_TRUNC | O_BINARY,
+				S_IRUSR | S_IWUSR);
 #endif /* Libburn_log_in_and_out_streaM */
 
 
@@ -2098,7 +2106,7 @@ int burn_dvd_write_session(struct burn_write_opts *o,
 		libdax_msgs_submit(libdax_messenger, d->global_index,
 				0x00020171,
 				LIBDAX_MSGS_SEV_NOTE, LIBDAX_MSGS_PRIO_HIGH,
-				"Closing BD-R with accidently open session",
+				"Closing BD-R with accidentally open session",
 				0, 0);
  		d->close_track_session(d, 3, 0); /* CLOSE SESSION, 110b */
 		d->state_of_last_session = 3; /* mark as complete session */
@@ -2520,7 +2528,7 @@ int burn_stdio_open_write(struct burn_drive *d, off_t start_byte,
 	if (fd >= 0)
 		fd = dup(fd); /* check validity and make closeable */
 	else
-		fd = open(d->devname, mode,
+		fd = open(d->devname, mode | O_BINARY,
                     S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH);
 	if (fd == -1) {
 		libdax_msgs_submit(libdax_messenger, d->global_index,
@@ -2575,29 +2583,41 @@ int burn_stdio_read_source(struct burn_source *source, char *buf, int bufsize,
 int burn_stdio_write(int fd, char *buf, int count, struct burn_drive *d, 
 			 int flag)
 {
-	int ret;
+	int ret = 0;
 	char *msg = NULL;
+	int todo, done, retries;
 
-	if (d->cancel)
+	if (d->cancel || count <= 0)
 		return 0;
+
+	todo = count;
+	done = 0;
+	for (retries = 0; todo > 0 && retries <= Libburn_stdio_write_retrieS;
+             retries++) {
 /*
 fprintf(stderr, "libburn_DEBUG: write(%d, %lX, %d)\n",
                 fd, (unsigned long) buf, count);
 */
-
-	ret = write(fd, buf, count);
-	if (ret != count) {
+		ret = write(fd, buf + done, todo);
+		if (ret < 0)
+	break;
+		done += ret;
+		todo -= ret;
+	}
+	if (done != count) {
 		BURN_ALLOC_MEM(msg, char, 160);
 
-		sprintf(msg,
-		  "Cannot write desired amount of data. write(2) returned %d.",
-		        ret);
+		sprintf(msg, "Cannot write desired amount of %d bytes.", count);
+		if (retries > 1)
+			sprintf(msg + strlen(msg), " Did %d retries. Last",
+			        retries - 1);
+		sprintf(msg + strlen(msg), " write(2) returned %d.", ret);
 		libdax_msgs_submit(libdax_messenger, d->global_index,
 			0x00020148,
 			LIBDAX_MSGS_SEV_SORRY, LIBDAX_MSGS_PRIO_HIGH,
 			msg, errno, 0);
 		d->cancel = 1;
-		return 0;
+		ret = 0; goto ex;
 	}
 ex:;
 	BURN_FREE_MEM(msg);
@@ -2662,10 +2682,11 @@ int burn_stdio_mmc_dummy_write(struct burn_drive *d, int start,
 /* ts A70911 */
 /* Flush stdio system buffer to physical device.
    @param flag bit0= do not report debug message (intermediate sync)
+               bit1= do fsync(2) unconditionally
 */
 int burn_stdio_sync_cache(int fd, struct burn_drive *d, int flag)
 {
-	int ret;
+	int ret, do_fsync;
 	char *msg = NULL;
 
 	if (fd < 0) {
@@ -2675,14 +2696,23 @@ int burn_stdio_sync_cache(int fd, struct burn_drive *d, int flag)
 			"Invalid file descriptor with stdio pseudo-drive",
 			0, 0);
 		d->cancel = 1;
-		return 0;
+		ret = 0; goto ex;
 	}
 	d->needs_sync_cache = 0;
-	if (!(flag & 1))
-		libdax_msgs_submit(libdax_messenger, -1, 0x00000002,
-			   LIBDAX_MSGS_SEV_DEBUG, LIBDAX_MSGS_PRIO_ZERO,
-			   "syncing cache (stdio fsync)", 0, 0);
-	ret = fsync(fd);
+	do_fsync = 0;
+	if (flag & 2)
+		do_fsync = 1;
+	else if (d->write_opts != NULL)
+		do_fsync = (d->write_opts->stdio_fsync_size >= 0);
+	if (do_fsync) {
+		if (!(flag & 1))
+			libdax_msgs_submit(libdax_messenger, -1, 0x00000002,
+				LIBDAX_MSGS_SEV_DEBUG, LIBDAX_MSGS_PRIO_ZERO,
+				"syncing cache (stdio fsync)", 0, 0);
+		ret = fsync(fd);
+	} else {
+		ret = 0;
+	}
 	if (ret != 0 && errno == EIO) {
 		BURN_ALLOC_MEM(msg, char, 160);
 
@@ -2694,7 +2724,7 @@ int burn_stdio_sync_cache(int fd, struct burn_drive *d, int flag)
 			LIBDAX_MSGS_SEV_SORRY, LIBDAX_MSGS_PRIO_HIGH,
 			msg, errno, 0);
 		d->cancel = 1;
-		return 0;
+		ret = 0; goto ex;
 	}
 	ret = 1;
 ex:;
@@ -2718,16 +2748,15 @@ int burn_stdio_slowdown(struct burn_drive *d, struct timeval *prev_time,
 			 int amount, int flag)
 {
 	struct timeval tnow;
-	struct timezone dummy_tz;
 	double to_wait;
 
 	if (flag & 1) {
-		gettimeofday(prev_time, &dummy_tz);
+		gettimeofday(prev_time, NULL);
 		return 1;
 	}
 	if(d->nominal_write_speed <= 0)
 		return 2;
-	gettimeofday(&tnow, &dummy_tz);
+	gettimeofday(&tnow, NULL);
 	to_wait = ( ((double) amount) / (double) d->nominal_write_speed ) - 
 		(double) ( tnow.tv_sec - prev_time->tv_sec ) -
 		(double) ( tnow.tv_usec - prev_time->tv_usec ) / 1.0e6
@@ -2735,7 +2764,7 @@ int burn_stdio_slowdown(struct burn_drive *d, struct timeval *prev_time,
 	if (to_wait >= 0.0001) {
 		usleep((int) (to_wait * 1000000.0));
 	}
-	gettimeofday(prev_time, &dummy_tz);
+	gettimeofday(prev_time, NULL);
 	return 1;
 }
 
@@ -2860,10 +2889,10 @@ ex:;
 		close(d->stdio_fd);
 	d->stdio_fd = -1;
 
-	/* update media state records */
+	/* update pseudo-media state records by re-grabbing */
 	burn_drive_mark_unready(d, 8);
+	burn_drive_grab_stdio(d, 1);
 
-	/* <<< d->busy = BURN_DRIVE_IDLE; */
 	return ret;
 }
 
@@ -3151,6 +3180,7 @@ int burn_random_access_write(struct burn_drive *d, off_t byte_address,
 				char *data, off_t data_count, int flag)
 {
 	int alignment = 0, start, upto, chunksize, err, fd = -1, ret;
+	int do_close = 0, getfl_ret;
 	char msg[81], *rpt;
 	struct buffer *buf = NULL, *buffer_mem = d->buffer;
 
@@ -3223,10 +3253,25 @@ int burn_random_access_write(struct burn_drive *d, off_t byte_address,
 			"Drive is busy on attempt to write random access",0,0);
 		{ret = 0; goto ex;}
 	}
-	if(d->drive_role != 1) {
-		fd = burn_stdio_open_write(d, byte_address, 2048, 0);
-		if (fd == -1)
-			{ret = 0; goto ex;}
+	if (d->drive_role != 1) {
+		if (d->stdio_fd >= 0) {
+			/* Avoid to have a read-only fd open */
+			getfl_ret = fcntl(d->stdio_fd, F_GETFL);
+			if (((O_RDWR | O_WRONLY | O_RDONLY) & getfl_ret) ==
+			    O_RDONLY) {
+				close(d->stdio_fd);
+				d->stdio_fd = -1;
+			}
+		}
+		if (d->stdio_fd >= 0) {
+			/* Avoid to have two fds open */
+			fd = d->stdio_fd;
+		} else {
+			fd = burn_stdio_open_write(d, byte_address, 2048, 0);
+			if (fd == -1)
+				{ret = 0; goto ex;}
+			do_close = 1;
+		}
 	}
 	d->cancel = 0;
 	d->busy = BURN_DRIVE_WRITING_SYNC;
@@ -3255,7 +3300,7 @@ int burn_random_access_write(struct burn_drive *d, off_t byte_address,
 		}
 		if (err == BE_CANCELLED) {
 			d->busy = BURN_DRIVE_IDLE;
-			if(fd >= 0)
+			if(fd >= 0 && do_close)
 				close(fd);
 			{ret = -(start * 2048 - byte_address); goto ex;}
 		}
@@ -3267,11 +3312,11 @@ int burn_random_access_write(struct burn_drive *d, off_t byte_address,
 		if(d->drive_role == 1)
 			d->sync_cache(d);
 		else
-			burn_stdio_sync_cache(fd, d, 0);
+			burn_stdio_sync_cache(fd, d, 2);
 		d->needs_sync_cache = 0;
 	}
 		
-	if(fd >= 0)
+	if(fd >= 0 && do_close)
 		close(fd);
 	d->buffer = buffer_mem;
 	d->busy = BURN_DRIVE_IDLE;
